@@ -464,6 +464,60 @@ class AdvancedCOCOtoYOLOMerger:
     - Proper class ID remapping for consistent training
     """
     
+    # Standard COCO category names (80 classes)
+    # These correspond to category IDs 1-80 in COCO (ID 0 is not used)
+    COCO_CATEGORY_NAMES = [
+        'person', 'bicycle', 'car', 'motorcycle', 'airplane', 'bus', 'train', 'truck', 'boat',
+        'traffic light', 'fire hydrant', 'stop sign', 'parking meter', 'bench', 'bird', 'cat',
+        'dog', 'horse', 'sheep', 'cow', 'elephant', 'bear', 'zebra', 'giraffe', 'backpack',
+        'umbrella', 'handbag', 'tie', 'suitcase', 'frisbee', 'skis', 'snowboard', 'sports ball',
+        'kite', 'baseball bat', 'baseball glove', 'skateboard', 'surfboard', 'tennis racket',
+        'bottle', 'wine glass', 'cup', 'fork', 'knife', 'spoon', 'bowl', 'banana', 'apple',
+        'sandwich', 'orange', 'broccoli', 'carrot', 'hot dog', 'pizza', 'donut', 'cake',
+        'chair', 'couch', 'potted plant', 'bed', 'dining table', 'toilet', 'tv', 'laptop',
+        'mouse', 'remote', 'keyboard', 'cell phone', 'microwave', 'oven', 'toaster', 'sink',
+        'refrigerator', 'book', 'clock', 'vase', 'scissors', 'teddy bear', 'hair drier', 'toothbrush'
+    ]
+    
+    @staticmethod
+    def _normalize_category_name(cat_name: str, cat_id: int) -> str:
+        """
+        Normalize category name from various formats to proper COCO category name.
+        
+        Handles:
+        - "class_X" format -> maps to COCO category name
+        - Numeric IDs -> maps to COCO category name
+        - Already proper names -> returns as-is
+        
+        Args:
+            cat_name: Category name from dataset
+            cat_id: Category ID from dataset
+            
+        Returns:
+            Normalized COCO category name
+        """
+        # If it's already a proper name (not class_X), return as-is
+        if not cat_name.startswith('class_'):
+            return cat_name
+        
+        # Try to extract numeric ID from "class_X" format
+        try:
+            # Extract number from "class_45" -> 45
+            class_num = int(cat_name.replace('class_', ''))
+            # Map to COCO category name (COCO IDs are 1-80, but HuggingFace uses 0-79)
+            if 0 <= class_num < len(AdvancedCOCOtoYOLOMerger.COCO_CATEGORY_NAMES):
+                return AdvancedCOCOtoYOLOMerger.COCO_CATEGORY_NAMES[class_num]
+        except (ValueError, IndexError):
+            pass
+        
+        # Try using category ID directly (COCO IDs are 1-80)
+        if 1 <= cat_id <= 80:
+            # COCO IDs are 1-indexed, array is 0-indexed
+            return AdvancedCOCOtoYOLOMerger.COCO_CATEGORY_NAMES[cat_id - 1]
+        
+        # Fallback: return original name
+        return cat_name
+    
     def __init__(self, output_dir: Path):
         """
         Initialize converter
@@ -564,6 +618,34 @@ class AdvancedCOCOtoYOLOMerger:
             if not img_source_path.exists():
                 continue
             
+            # Get annotations first to check if image will have valid labels after filtering
+            img_width, img_height = img_info['width'], img_info['height']
+            annotations = coco.loadAnns(coco.getAnnIds(imgIds=[img_id]))
+            
+            # Filter annotations and count valid ones
+            filtered_count = 0
+            valid_annotations = []
+            
+            for ann in annotations:
+                if 'bbox' in ann:
+                    coco_cat_id = ann['category_id']
+                    
+                    # Map COCO category_id to sequential YOLO class_id
+                    if coco_cat_id not in class_id_map:
+                        filtered_count += 1
+                        continue  # Skip unknown classes (filtered by label strategy)
+                    
+                    yolo_class_id = class_id_map[coco_cat_id]
+                    bbox = self.convert_bbox_to_yolo(img_width, img_height, ann['bbox'], use_xyxy_format=use_xyxy_format)
+                    label_str = f"{yolo_class_id} {' '.join(f'{v:.6f}' for v in bbox)}\n"
+                    valid_annotations.append(label_str)
+            
+            # For base_dataset strategy: if image has no valid annotations after filtering, exclude it
+            if label_strategy == 'base_dataset' and len(valid_annotations) == 0:
+                if filtered_count > 0:
+                    logger.debug(f"   ⏭️  Excluding image {img_filename} (no valid labels after filtering)")
+                continue  # Skip this image entirely
+            
             # Symlink image (no copying!)
             img_dest_path = output_images_dir / img_filename
             try:
@@ -575,31 +657,14 @@ class AdvancedCOCOtoYOLOMerger:
                 continue
             
             # Create YOLO label file
-            img_width, img_height = img_info['width'], img_info['height']
-            annotations = coco.loadAnns(coco.getAnnIds(imgIds=[img_id]))
-            
             label_file = output_labels_dir / f"{img_filename.rsplit('.', 1)[0]}.txt"
-            filtered_count = 0
             with open(label_file, 'w') as f:
                 seen_labels = set()
-                for ann in annotations:
-                    if 'bbox' in ann:
-                        coco_cat_id = ann['category_id']
-                        
-                        # Map COCO category_id to sequential YOLO class_id
-                        if coco_cat_id not in class_id_map:
-                            filtered_count += 1
-                            continue  # Skip unknown classes (filtered by label strategy)
-                        
-                        yolo_class_id = class_id_map[coco_cat_id]
-                        
-                        bbox = self.convert_bbox_to_yolo(img_width, img_height, ann['bbox'], use_xyxy_format=use_xyxy_format)
-                        label_str = f"{yolo_class_id} {' '.join(f'{v:.6f}' for v in bbox)}\n"
-                        
-                        if label_str not in seen_labels:
-                            f.write(label_str)
-                            seen_labels.add(label_str)
-                            label_count += 1
+                for label_str in valid_annotations:
+                    if label_str not in seen_labels:
+                        f.write(label_str)
+                        seen_labels.add(label_str)
+                        label_count += 1
             
             if filtered_count > 0 and label_strategy == 'base_dataset':
                 logger.debug(f"   ⏭️  Filtered {filtered_count} annotations (labels not in base dataset)")
@@ -710,6 +775,10 @@ class AdvancedCOCOtoYOLOMerger:
                                 cat_name = cat.get('name') or cat.get('category_name', '')
                                 cat_id = cat.get('id') or cat.get('category_id', 0)
                                 
+                                # Normalize category name (convert "class_X" to proper COCO name)
+                                if cat_name:
+                                    cat_name = AdvancedCOCOtoYOLOMerger._normalize_category_name(cat_name, cat_id)
+                                
                                 if cat_name:
                                     cat_name_normalized = ' '.join(cat_name.lower().strip().split())
                                     all_categories[cat_name] = (cat_id, base_dataset_index)
@@ -769,6 +838,10 @@ class AdvancedCOCOtoYOLOMerger:
                                 cat_name = cat.get('name') or cat.get('category_name', '')
                                 cat_id = cat.get('id') or cat.get('category_id', 0)
                                 
+                                # Normalize category name (convert "class_X" to proper COCO name)
+                                if cat_name:
+                                    cat_name = AdvancedCOCOtoYOLOMerger._normalize_category_name(cat_name, cat_id)
+                                
                                 if cat_name:
                                     # Normalize name for case-insensitive comparison (remove extra whitespace)
                                     cat_name_normalized = ' '.join(cat_name.lower().strip().split())
@@ -893,6 +966,9 @@ class AdvancedCOCOtoYOLOMerger:
                     
                     for cat in categories:
                         cat_name = cat['name']
+                        cat_id = cat.get('id', 0)
+                        # Normalize category name (convert "class_X" to proper COCO name)
+                        cat_name = AdvancedCOCOtoYOLOMerger._normalize_category_name(cat_name, cat_id)
                         cat_name_normalized = cat_name.lower().strip()
                         
                         # Case-insensitive matching for mapping
