@@ -98,6 +98,8 @@ class YOLOTrainer:
         weight_decay: float = 0.0005,
         warmup_epochs: int = 3,
         patience: int = 50,
+        training_mode: str = "fine_tune",  # NEW: "fine_tune" or "from_scratch"
+        resume_checkpoint: Optional[str] = None,  # NEW: Path to checkpoint for resuming
         # Augmentation parameters
         hsv_h: float = 0.015,
         hsv_s: float = 0.7,
@@ -126,6 +128,8 @@ class YOLOTrainer:
             weight_decay: Weight decay factor
             warmup_epochs: Number of warmup epochs
             patience: Early stopping patience
+            training_mode: Training mode - "fine_tune" (use pretrained weights) or "from_scratch" (no pretrained weights)
+            resume_checkpoint: Path to checkpoint (last.pt) for resuming interrupted training
             hsv_h: HSV-Hue augmentation (0-1)
             hsv_s: HSV-Saturation augmentation (0-1)
             hsv_v: HSV-Value augmentation (0-1)
@@ -148,6 +152,8 @@ class YOLOTrainer:
         self.weight_decay = weight_decay
         self.warmup_epochs = warmup_epochs
         self.patience = patience
+        self.training_mode = training_mode
+        self.resume_checkpoint = resume_checkpoint
         
         # Augmentation parameters
         self.hsv_h = hsv_h
@@ -319,40 +325,77 @@ class YOLOTrainer:
             Training results dictionary
         """
         
-        logger.info(f"🤖 Initializing {self.model_variant} model...")
-        
-        # Try different model naming conventions (yolov11l vs yolo11l)
-        # YOLOv11 models use 'yolo11' not 'yolov11'
-        model_variants_to_try = []
-        
-        # For v11 models, try yolo11 first (correct naming)
-        if 'v11' in self.model_variant:
-            alternative = self.model_variant.replace('yolov11', 'yolo11')
-            model_variants_to_try = [alternative, self.model_variant]
+        # Check if we're resuming from a checkpoint
+        if self.resume_checkpoint:
+            logger.info(f"🔄 RESUMING training from checkpoint: {self.resume_checkpoint}")
+            checkpoint_path = Path(self.resume_checkpoint)
+            
+            if not checkpoint_path.exists():
+                raise FileNotFoundError(f"Resume checkpoint not found: {self.resume_checkpoint}")
+            
+            # Load the checkpoint directly
+            logger.info(f"📦 Loading checkpoint: {checkpoint_path}")
+            self.model = YOLO(str(checkpoint_path))
+            logger.info(f"✅ Checkpoint loaded successfully - will resume training")
+            
         else:
-            model_variants_to_try = [self.model_variant]
-        
-        last_error = None
-        for variant in model_variants_to_try:
-            try:
-                model_path = f'{variant}.pt'
-                logger.info(f"⬇️ Attempting to load: {model_path}")
-                self.model = YOLO(model_path)
-                logger.info(f"✅ Loaded model: {model_path}")
-                break
-            except Exception as e:
-                last_error = e
-                logger.warning(f"⚠️ Failed to load {variant}.pt: {str(e)}")
-                continue
-        
-        if self.model is None:
-            raise FileNotFoundError(
-                f"Model not found after trying: {', '.join([f'{v}.pt' for v in model_variants_to_try])}\n"
-                f"Valid models: yolov8n/s/m/l/x, yolov9t/s/m/c/e, yolov10n/s/m/b/l/x, yolo11n/s/m/l/x\n"
-                f"Last error: {last_error}"
-            )
+            # Normal model initialization (not resuming)
+            logger.info(f"🤖 Initializing {self.model_variant} model...")
+            
+            # Determine file extension based on training mode
+            if self.training_mode == "from_scratch":
+                file_ext = ".yaml"  # Load architecture only (random initialization)
+                logger.info("📄 Loading model architecture from YAML (random weight initialization)")
+                logger.info("   No pretrained weights will be loaded - training truly from scratch")
+            else:
+                file_ext = ".pt"    # Load pretrained weights
+                logger.info("📦 Loading pretrained model weights from .pt file")
+                logger.info("   Model will start with COCO-trained weights (fine-tuning)")
+            
+            # Try different model naming conventions (yolov11l vs yolo11l)
+            # YOLOv11 models use 'yolo11' not 'yolov11'
+            model_variants_to_try = []
+            
+            # For v11 models, try yolo11 first (correct naming)
+            if 'v11' in self.model_variant:
+                alternative = self.model_variant.replace('yolov11', 'yolo11')
+                model_variants_to_try = [alternative, self.model_variant]
+            else:
+                model_variants_to_try = [self.model_variant]
+            
+            last_error = None
+            for variant in model_variants_to_try:
+                try:
+                    model_path = f'{variant}{file_ext}'
+                    logger.info(f"⬇️ Attempting to load: {model_path}")
+                    
+                    # CRITICAL: For from_scratch mode, we must ensure no pretrained weights are loaded
+                    # Even when loading .yaml, YOLO() might auto-load matching .pt file if it exists
+                    # So we explicitly pass task to force architecture-only loading
+                    if self.training_mode == "from_scratch":
+                        # Load YAML with explicit task to prevent auto-loading of .pt weights
+                        self.model = YOLO(model_path, task='detect')
+                        logger.info(f"✅ Loaded model architecture: {model_path} (no pretrained weights)")
+                    else:
+                        # Load pretrained .pt file normally
+                        self.model = YOLO(model_path)
+                        logger.info(f"✅ Loaded pretrained model: {model_path}")
+                    break
+                except Exception as e:
+                    last_error = e
+                    logger.warning(f"⚠️ Failed to load {variant}{file_ext}: {str(e)}")
+                    continue
+            
+            if self.model is None:
+                raise FileNotFoundError(
+                    f"Model not found after trying: {', '.join([f'{v}{file_ext}' for v in model_variants_to_try])}\n"
+                    f"Valid models: yolov8n/s/m/l/x, yolov9t/s/m/c/e, yolov10n/s/m/b/l/x, yolo11n/s/m/l/x\n"
+                    f"File extension: {file_ext} (training_mode={self.training_mode})\n"
+                    f"Last error: {last_error}"
+                )
         
         logger.info(f"🚀 Starting training for {self.epochs} epochs...")
+        logger.info(f"🎯 Training mode: {self.training_mode}")
         
         # Detect available device (CUDA, MPS, or CPU)
         if torch.cuda.is_available():
@@ -372,38 +415,68 @@ class YOLOTrainer:
         workers = self._get_optimal_workers()
         logger.info(f"🔧 Using {workers} DataLoader workers")
         
+        # Determine freeze parameter and pretrained setting based on training mode
+        # For from_scratch: freeze=[] means no layers are frozen (train all layers)
+        #                   pretrained=False prevents auto-downloading pretrained weights
+        # For fine_tune: freeze=None uses Ultralytics defaults (may freeze some layers)
+        #                pretrained=True allows loading pretrained weights
+        if self.training_mode == "from_scratch":
+            freeze_layers = []  # Don't freeze ANY layers
+            use_pretrained = False  # CRITICAL: Prevent auto-download of pretrained weights
+            logger.info("🔓 All layers unfrozen - training all weights from scratch")
+            logger.info("🚫 Pretrained weight loading DISABLED - no auto-download")
+            logger.info("⚠️  Note: Ultralytics always freezes DFL layer by design (16 params)")
+            logger.info("   This is intentional and won't affect training quality")
+        else:
+            freeze_layers = None  # Use Ultralytics defaults for fine-tuning
+            use_pretrained = True  # Allow pretrained weights for fine-tuning
+            logger.info("🔒 Using default layer freezing strategy for fine-tuning")
+            logger.info("✅ Pretrained weight loading ENABLED")
+        
         # Train the model
-        self.results = self.model.train(
-            data=self.dataset_path,
-            epochs=self.epochs,
-            batch=self.batch_size,
-            imgsz=self.img_size,
-            lr0=self.learning_rate,
-            momentum=self.momentum,
-            weight_decay=self.weight_decay,
-            warmup_epochs=self.warmup_epochs,
-            patience=self.patience,
-            project=str(self.output_dir),
-            name="train",
-            exist_ok=True,
-            verbose=True,
-            device=device,
-            workers=workers,
-            save=True,
-            save_period=10,
-            augment=True,
+        train_params = {
+            "data": self.dataset_path,
+            "epochs": self.epochs,
+            "batch": self.batch_size,
+            "imgsz": self.img_size,
+            "lr0": self.learning_rate,
+            "momentum": self.momentum,
+            "weight_decay": self.weight_decay,
+            "warmup_epochs": self.warmup_epochs,
+            "patience": self.patience,
+            "project": str(self.output_dir),
+            "name": "train",
+            "exist_ok": True,
+            "verbose": True,
+            "device": device,
+            "workers": workers,
+            "save": True,
+            "save_period": 10,
+            "augment": True,
+            "amp": False,  # Disable automatic mixed precision to avoid CUBLAS_STATUS_INTERNAL_ERROR
             # Augmentation parameters
-            hsv_h=self.hsv_h,
-            hsv_s=self.hsv_s,
-            hsv_v=self.hsv_v,
-            degrees=self.degrees,
-            translate=self.translate,
-            scale=self.scale,
-            flipud=self.flipud,
-            fliplr=self.fliplr,
-            mosaic=self.mosaic,
-            mixup=self.mixup,
-        )
+            "hsv_h": self.hsv_h,
+            "hsv_s": self.hsv_s,
+            "hsv_v": self.hsv_v,
+            "degrees": self.degrees,
+            "translate": self.translate,
+            "scale": self.scale,
+            "flipud": self.flipud,
+            "fliplr": self.fliplr,
+            "mosaic": self.mosaic,
+            "mixup": self.mixup,
+        }
+        
+        # Add resume parameter if resuming from checkpoint
+        if self.resume_checkpoint:
+            train_params["resume"] = True  # Tell Ultralytics to resume training
+            logger.info("🔄 Resume mode enabled - training will continue from checkpoint state")
+        else:
+            # Only set these for non-resume training
+            train_params["pretrained"] = use_pretrained  # Control pretrained weights based on training_mode
+            train_params["freeze"] = freeze_layers  # Control layer freezing based on training_mode
+        
+        self.results = self.model.train(**train_params)
         
         logger.info("✅ Training completed!")
         
