@@ -177,11 +177,10 @@ def main():
         resume_checkpoint = config.get('resume_checkpoint', None)  # Path to checkpoint for resuming
         # Label management strategy
         label_strategy = config.get('labelStrategy', 'merge_all')  # 'merge_all' or 'base_dataset'
-        base_dataset_index = config.get('baseDatasetIndex', None)  # Index of base dataset (0-based)
         
         log_progress(f"🔍 DEBUG: After extraction, epochs = {epochs}")
         log_progress(f"🔍 DEBUG: training_mode = {training_mode}")
-        log_progress(f"🔍 DEBUG: label_strategy = {label_strategy}, base_dataset_index = {base_dataset_index}")
+        log_progress(f"🔍 DEBUG: label_strategy = {label_strategy}")
         
         if resume_checkpoint:
             log_progress(f"🔄 Resume mode: Will continue training from checkpoint: {resume_checkpoint}")
@@ -241,45 +240,56 @@ def main():
         # Preprocess datasets (convert COCO standard to flat format)
         preprocessed_datasets = preprocess_datasets(datasets_found, WORKING_DIR)
         
-        # For base_dataset strategy, validate that we have the required information
-        actual_base_index = base_dataset_index
+        # For base_dataset strategy, find the base dataset by name
+        actual_base_index = None
         if label_strategy == 'base_dataset':
-            # Get the base dataset name from config if available
             base_dataset_name = config.get('baseDatasetName', None)
-            datasets_config = config.get('datasets', [])
             
-            log_progress(f"🔍 DEBUG: Label strategy is 'base_dataset', base_dataset_index = {base_dataset_index}")
-            log_progress(f"🔍 DEBUG: baseDatasetName = {base_dataset_name}")
-            log_progress(f"🔍 DEBUG: Found {len(datasets_found)} datasets, preprocessed {len(preprocessed_datasets)} datasets")
-            
-            # Validate that we have base dataset information
-            if actual_base_index is None:
+            if not base_dataset_name:
                 error_msg = (
-                    f"❌ Configuration error: label_strategy='base_dataset' but baseDatasetIndex is None. "
-                    f"This indicates a bug in the UI or agent configuration. "
-                    f"baseDatasetName={base_dataset_name}, datasets_config length={len(datasets_config)}. "
+                    f"❌ Configuration error: label_strategy='base_dataset' but baseDatasetName is missing. "
                     f"Training cannot proceed without knowing which dataset to use as the base."
                 )
                 log_progress(error_msg, status="error")
                 raise ValueError(error_msg)
             
-            # Validate index is in range
-            if actual_base_index < 0 or actual_base_index >= len(preprocessed_datasets):
+            log_progress(f"🔍 DEBUG: Label strategy is 'base_dataset', looking for: {base_dataset_name}")
+            log_progress(f"🔍 DEBUG: Found {len(datasets_found)} datasets, preprocessed {len(preprocessed_datasets)} datasets")
+            
+            # Sanitize name to match folder naming (same as agent does)
+            safe_name = base_dataset_name.replace('/', '-').replace(' ', '-').lower()
+            log_progress(f"🔍 DEBUG: Sanitized base dataset name: {safe_name}")
+            
+            # Try to find matching dataset in discovered datasets
+            for i, ds_info in enumerate(datasets_found):
+                ds_name = ds_info.get('name', '')
+                ds_path_name = Path(ds_info.get('path', '')).name.lower()
+                if safe_name in ds_name.lower() or safe_name in ds_path_name:
+                    actual_base_index = i
+                    log_progress(f"🔍 DEBUG: Found base dataset at index {i}: {ds_name} (path: {ds_path_name})")
+                    break
+            
+            # If not found in discovered datasets, try matching against preprocessed dataset paths
+            if actual_base_index is None:
+                for i, (ds_path, ds_format) in enumerate(preprocessed_datasets):
+                    ds_path_name = Path(ds_path).name.lower()
+                    if safe_name in ds_path_name:
+                        actual_base_index = i
+                        log_progress(f"🔍 DEBUG: Found base dataset at preprocessed index {i}: {ds_path_name}")
+                        break
+            
+            # Validate we found the base dataset
+            if actual_base_index is None:
+                available_names = [ds.get('name', Path(ds.get('path', '')).name) for ds in datasets_found]
                 error_msg = (
-                    f"❌ Configuration error: baseDatasetIndex={actual_base_index} is out of range. "
-                    f"Valid range: 0-{len(preprocessed_datasets)-1}. "
-                    f"Found {len(preprocessed_datasets)} preprocessed datasets. "
-                    f"This may indicate a mismatch between the selected datasets and discovered datasets."
+                    f"❌ Configuration error: Could not find base dataset '{base_dataset_name}' (sanitized: '{safe_name}'). "
+                    f"Found {len(datasets_found)} discovered datasets. "
+                    f"Available dataset names: {available_names}. "
+                    f"Training cannot proceed without identifying the base dataset."
                 )
                 log_progress(error_msg, status="error")
                 raise ValueError(error_msg)
             
-            # If we have an index, try to get the name for verification
-            if actual_base_index is not None and datasets_config and actual_base_index < len(datasets_config):
-                if not base_dataset_name:
-                    base_dataset_name = datasets_config[actual_base_index].get('name', '')
-                    log_progress(f"🔍 DEBUG: Got base dataset name from config: {base_dataset_name}")
-            
             # Log all datasets for debugging
             for i, (ds_path, ds_format) in enumerate(preprocessed_datasets):
                 marker = " <-- BASE" if i == actual_base_index else ""
@@ -287,11 +297,9 @@ def main():
         
         # Merge and convert datasets to YOLO format
         if label_strategy == 'base_dataset' and actual_base_index is not None:
-            log_progress(f"⚡ Merging datasets with base dataset strategy (using dataset index {actual_base_index})...")
+            log_progress(f"⚡ Merging datasets with base dataset strategy (using dataset at index {actual_base_index})...")
         else:
             log_progress("⚡ Merging datasets and converting to YOLO format...")
-            if label_strategy != 'merge_all':
-                log_progress(f"⚠️  Warning: label_strategy='{label_strategy}' but base_dataset_index is None, defaulting to merge_all")
         output_dir = WORKING_DIR / "yolo_dataset"
         merger = AdvancedCOCOtoYOLOMerger(output_dir=output_dir)
         
